require 'spec_helper'

module ReservationRecommendationsTest
  # This spec will not be run on all branch builds
  describe 'ReservationRecommendations refreshReservationRecommendationSource' do
    include_context 'request metadata'

    # The following e2e test does the following:
    # Kicks off an artifact refresh via a refreshReservationRecommendationSource query in the RRS
    # Polls the state of the job via generateRecommendationsStatus query until it gets to a COMPLETED status
    # Verify the job gets into a RUNNING state (during generateRecommendationsStatus polling)
    # After job is started, queries artifactJobStatuses in the savings plan scheduler, to verify new job was created after refresh request was made
    # Grabs job id from artifactJobStatuses response
    # While job is running, polls the sp scheduler service via node request on ArtifactRefreshTaskStatus, to verify job gets into FLEXREPORT_SETUP state
    # After verifying job gets to COMPLETED state via generateRecommendationsStatus response
    # Verify sp scheduler job has also got to COMPLETED state, get finishedAt of last COMPLETED job in artifactJobStatuses array
    # Make updateReservationRecommendations request in RRS, verify asOfDate in response matches finishedAt of last COMPLETED job in artifactJobStatuses array
    # Make reservationRecommendations query in RRS, verify at least 1 recommendation is returned
    # Make reservationRecommendationUsage query in RRS, verify valid usage data is returned for all recommendations
    it 'Verify refreshReservationRecommendationSource e2e', manual: true do
      date = Time.new
      date_time_before_refresh = Time.new
      if TEST_DATA_ENV == 'preprod'
        # This will work for preprod
        billing_account = "crn:#{customerId}:aws-account/#{REDSHIRT_PREPROD[:owner_id]}"
        customer_info = {
          accountId: REDSHIRT_PREPROD[:aws_billing_id],
          customerId: REDSHIRT_PREPROD[:customer_id],
          organizationId: REDSHIRT_PREPROD[:organization_id]
        }
      else
        # This will work for staging and prod
        billing_account = "crn:#{customerId}:aws-account/#{REDSHIRT[:owner_id]}"
        customer_info = {
          accountId: REDSHIRT[:aws_billing_id],
          customerId: REDSHIRT[:customer_id],
          organizationId: REDSHIRT[:organization_id]
        }
      end

      wrap_request do
        variables = {
          'billingAccount': billing_account
        }
        graphql url(service: service),
          format_request(
            :refreshReservationRecommendationSource,
            service: service
          ),
          variables, headers
        expect_success_graphql
        check(:refreshReservationRecommendationSource, service: service)
      end
      expect(json_body[:data][:refreshReservationRecommendationSource]).to eq(billing_account)

      wait_time_increment = 10
      retry_attempts = 120
      time_for_complete = wait_time_increment * (retry_attempts - 1)
      puts 'getting initial status for refresh job'
      wrap_request do
        variables = {
          billingAccount: billing_account
        }
        graphql url(service: service),
          format_request(
            :generateRecommendationsStatus,
            service: service
          ),
          variables, headers
        expect_success_graphql
      end
      puts JSON::pretty_generate(json_body[:data])
      if json_body[:data][:generateRecommendationsStatus]
        quote_status = json_body[:data][:generateRecommendationsStatus][:status]
        sleep(wait_time_increment)

        # verify new job was created via sp scheduler after refresh request was made
        wrap_request do
          variables = {
            artifactType: 'DOT',
            customerInfo: customer_info,
            sortRules: [{field: 'createdAt',
                         direction: 'DESC'}],
          }
          graphql url(service: sp_scheduler_service),
            format_request(
              :artifactJobStatuses,
              service: sp_scheduler_service
            ),
            variables, headers
          expect_success_graphql
          check(:artifactJobStatuses, service: sp_scheduler_service)
        end
        job_array = json_body[:data][:artifactJobStatuses][:edges]
        scheduler_job_id = ""

        puts "Time before refresh : #{date_time_before_refresh}"
        puts "Scheduled jobs list for the last 3 days"
        job_array.each {|job|
          if Time.parse(job[:node][:createdAt]) >= (Time.now - (3*24*60*60))
            puts "#{job[:node][:id]}, #{job[:node][:createdAt]}, #{job[:node][:status]}"
            if Time.parse(job[:node][:createdAt]) >= date_time_before_refresh
              scheduler_job_id = job[:node][:id]
            end
          end
        }

        fail("Job is not scheduled in SPA") if scheduler_job_id.empty?
        puts "Scheduler Job ID : #{scheduler_job_id}"
        running_state_ran = false
        if quote_status == 'FAILED'
          raise Exception, "recommendation creation job failed: #{json_body}"
        elsif quote_status == 'RUNNING'
          running_state_ran = true
        end

        puts "DOT notification is sent to the preprod SQS queue : https://sqs.us-east-2.amazonaws.com/924758434213/reservation_recommendation_dot_notification_queue_use2_preprod_test.fifo"
        puts "Copy and send the same message to localstack SQS queue in the PR namespace"
        puts "Sample Command :- "
        puts "awslocal sqs send-message --queue-url http://localhost:4566/000000000000/rrs-pr-test-dot-queue.fifo --message-body='{\"cloudType\":\"AWS\",\"flexReportDatasetName\":\"DOT\",\"taskCrn\":\"crn:57534:spr-internal-task/57439fd4-9e8b-47f4-8f73-f5cd65974e83\",\"customerId\":\"57534\",\"status\":\"COMPLETE\",\"billingAccount\":\"crn:57534:aws-account/844584796563\",\"dotVersionId\":\"crn:57534:DOT/57439fd4-9e8b-47f4-8f73-f5cd65974e83-20240425095614\",\"manifestPath\":\"s3://cht-sp-allocation-use2-preprod-test/dot/57534/3/dot-manifest.json\",\"creationDateTime\":\"2024-04-25T09:56:23Z\"}' --message-group-id='test'"
        puts "Waiting for a minute to copy the same message and send the SQS message to localstack sqs"
        sleep(60)

        attempts = 0
        wait_time = wait_time_increment
        while quote_status != 'COMPLETED' && attempts < retry_attempts
          puts "waiting #{wait_time} seconds for create quote completion (status: #{quote_status})"
          # poll the state of the job via generateRecommendationsStatus query until it gets to a COMPLETED status
          wrap_request do
            variables = {
               billingAccount: billing_account
            }
            graphql url(service: service),
              format_request(
                :generateRecommendationsStatus,
                service: service
              ),
              variables, headers
            expect_success_graphql
            check(:generateRecommendationsStatus, service: service)
          end
          quote_status = json_body[:data][:generateRecommendationsStatus][:status]
          # Verify the job gets into a RUNNING state
          # fail fast if quote status goes to FAILED
          if quote_status == 'FAILED'
            raise Exception, "recommendation creation job failed: #{json_body}"
          elsif quote_status == 'RUNNING'
            running_state_ran = true
          end

          # verify scheduler job gets into FLEXREPORT_SETUP state during job run
          wrap_request do
            variables = {}
            graphql url(service: sp_scheduler_service),
              format_request(
                :node,
                service: sp_scheduler_service,
                crn: scheduler_job_id,
                type: 'ArtifactRefreshTaskStatus',
                fields:  'id, status'
              ),
              variables, headers
            expect_success_graphql
            check(:node, service: sp_scheduler_service)
          end
          scheduler_quote_status = json_body[:data][:node][:status]
          puts "scheduler job id: #{scheduler_job_id}, status: #{scheduler_quote_status}"
          if scheduler_quote_status == 'FAILED'
            raise Exception, "recommendation creation job failed: #{json_body}"
          end
          break if scheduler_quote_status == 'COMPLETED'
          attempts += 1
          sleep(wait_time_increment)
          wait_time += wait_time_increment
        end

        # verify job gets to COMPLETED state in RRS via generateRecommendationsStatus response
        wrap_request do
          variables = {
            billingAccount: billing_account
          }
          graphql url(service: service),
            format_request(
              :generateRecommendationsStatus,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:generateRecommendationsStatus, service: service)
        end
        finished_at_date = json_body[:data][:generateRecommendationsStatus][:finishedAt]
        expect(json_body[:data][:generateRecommendationsStatus][:status]).to eq('COMPLETED')
        expect(finished_at_date).to be > json_body[:data][:generateRecommendationsStatus][:startedAt]

        # verify job gets to COMPLETED state in sp scheduler via ArtifactRefreshTaskStatus response
        # TBD

        # get finishedAt date of last COMPLETED job in artifactJobStatuses array
        wrap_request do
          variables = {
            artifactType: 'DOT',
            customerInfo: customer_info,
            filterRules: [
              {field: "status",
               filterType: "EQ",
               filterValues: ["COMPLETED"]}
            ],
            sortRules: [{field: 'createdAt',
                         direction: 'DESC'}],
          }
          graphql url(service: sp_scheduler_service),
            format_request(
              :artifactJobStatuses,
              service: sp_scheduler_service
            ),
            variables, headers
          expect_success_graphql
          check(:artifactJobStatuses, service: sp_scheduler_service)
        end
        last_completed_artifact = json_body[:data][:artifactJobStatuses][:edges][0][:node][:finishedAt]

        wrap_request do
          variables = {
            analysisInput: {
              awsAnalysisInput: {
                applicationScope: 'Regional',
                billingAccountId: "crn:#{customerId}:aws-account/#{REDSHIRT[:owner_id]}",
                linkedAccounts: [
                  "crn:#{customerId}:aws-account/#{REDSHIRT[:owner_id]}",
                  "crn:#{customerId}:aws-account/#{REDSHIRT_2[:owner_id]}",
                  "crn:#{customerId}:aws-account/#{REDSHIRT_3[:owner_id]}"
                ],
                lookBackPeriod: {
                    period: 'Last30Days',
                },
                paymentTypes: [
                    'NoUpfront'
                ],
                recommendationOption: {
                    isDefault: true
                },
                regions: [],
                services: [],
                term: 'ThreeYear'
                }
            },
            cloud: 'AWS'
          }
          graphql url(service: service),
            format_request(
              :updateReservationRecommendations,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:updateReservationRecommendations, service: service)
        end
        job_id = json_body[:data][:updateReservationRecommendations][:id]
        # verify asOfDate in response matches finishedAt of last COMPLETED job in artifactJobStatuses array
        as_of_date = json_body[:data][:updateReservationRecommendations][:sourceArtifactStatus][:asOfDate]
        expect(as_of_date).to eq(last_completed_artifact)

        # verify recommendations are available post artifact refresh/update
        wrap_request do
          variables = {
            cloud: 'AWS',
            scopeInput: {
              awsJobId: job_id
            }
          }
          graphql url(service: service),
            format_request(
              :reservationRecommendations,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:reservationRecommendations, service: service)
          expect(json_body[:data][:reservationRecommendations][:edges].length).to be > 0
        end

        # verify recommendations have valid usage post artifact refresh/update
        json_body[:data][:reservationRecommendations][:edges] .each {|rec|
          recommendation_id = rec[:node][:id]
          wrap_request do
            variables = {
              recommendationId: recommendation_id
            }
            graphql url(service: service),
              format_request(
                :reservationRecommendationUsage,
                service: service
              ),
              variables, headers
            expect_success_graphql
            check(:reservationRecommendationUsage, service: service)
            usage_data_array =  json_body[:data][:reservationRecommendationUsage][:usageData]
            usage_array_sum = usage_data_array.inject(0) { |sum, i| sum + i }
            avg_usage_value =  usage_array_sum / usage_data_array.length
            expect(avg_usage_value).to be > 0
          end
        }
      end
    end
  end
end
