require 'spec_helper'

module ReservationRecommendationsTest
  describe 'ReservationRecommendations reservationRecommendationUsage' do
    include_context 'request metadata'
    dynamodb_usage_types = ["ReadCapacityUnit-Hrs", "WriteCapacityUnit-Hrs"]

    RSpec.shared_examples 'Usage details tests' do |period, term, payment_type|
      it 'reservationRecommendationUsage returns expected response and corresponds to recommendations summary values' do
        services = [
          'Redshift',
          'EC2',
          'RDS',
          'ElastiCache',
          'OpenSearch',
          'DynamoDB'
        ]
        job_id = get_update_job_id(period, payment_type, term, services)
        wrap_request do
          variables = {
            cloud: 'AWS',
            scopeInput: {
              awsJobId: job_id
            }
          }
          graphql url(service: service),
            format_request(
              :reservationRecommendations,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:reservationRecommendations, service: service)
        end
        recommendations_array = json_body[:data][:reservationRecommendations][:edges]
        recommendation_presence_check(recommendations_array, 'AWS')

        recommendations_array.each {|rec|
          recommendations_id = rec[:node][:id]
          wrap_request do
            variables = {}
            graphql url(service: service),
              format_request(
                  :node,
                  service: service,
                  crn: recommendations_id,
                  type: 'AwsReservationRecommendation',
                  fields: 'id,
                          accountId,
                          accountName,
                          currency,
                          resourceProperties {
                            region,
                            serviceType,
                            tenancy,
                            platform,
                            availabilityZone,
                            instanceType,
                            usageType
                          },
                          savingsProperties {
                            analysisLookBackPeriod,
                            recommendedQuantity,
                            upfrontCost,
                            monthlyPayment,
                            savings,
                            monthlySavings,
                            annualSavings,
                            reservationOrderTerm,
                            paymentType,
                            currentCoverage,
                            newCoverage
                          }'
                ),
                variables, headers
            expect_success_graphql
            check(:node, service: service)
          end
          node_response = json_body[:data][:node]
          service_type = node_response[:resourceProperties][:serviceType]

          if service_type == "DynamoDB"
            expect(node_response[:resourceProperties][:instanceType]).to eq(nil)
            expect(dynamodb_usage_types).to include(node_response[:resourceProperties][:usageType])
          else
            expect(node_response[:resourceProperties][:usageType]).to eq(nil)
            expect(node_response[:resourceProperties][:instanceType]).to_not eq(nil)
            expect(dynamodb_usage_types).to_not include(node_response[:resourceProperties][:instanceType])
          end

          recommended_quantity = json_body[:data][:node][:savingsProperties][:recommendedQuantity]
          expect(recommended_quantity).to be > 0

          wrap_request do
            variables = {
                recommendationId: recommendations_id
            }
            graphql url(service: service),
              format_request(
                :reservationRecommendationUsage,
                service: service
              ),
              variables, headers
            expect_success_graphql
            check(:reservationRecommendationUsage, service: service)
          end
          usageResponse = json_body[:data][:reservationRecommendationUsage]

          if period == 'Last90Days'
            lookBackPeriod = 90
          elsif period == 'Last60Days'
            lookBackPeriod = 60
          elsif period == 'Last30Days'
            lookBackPeriod = 30
          else
            lookBackPeriod = 7
          end
          consumption_date_range = (Date.parse(usageResponse[:endDate]) - Date.parse(usageResponse[:startDate])).to_i + 1
          expect(consumption_date_range).to eq(lookBackPeriod)

          usage_arr = usageResponse[:usageData]
          usage_arr_sum = usage_arr.inject(0) { |sum, i| sum + i }
          avg_usage_value =  (usage_arr_sum / usage_arr.length).round(1)
          expect(avg_usage_value).to be > 0

          # It is possible to get a recommendation greater than the highest usage and a recommendation below the minimum/average usage,
          # however they will never be more than 1 recommendation unit higher or lower (unit is always 1 except for dynamodb which is 100)
          # Accounting for this in the below tests
          if service_type == "DynamoDB"
            avg_usage_value = (avg_usage_value / 100).round() * 100
            expect(recommended_quantity).to be < usage_arr.max() + 100
            expect(recommended_quantity).to be > usage_arr.min() - 100
          else
            expect(recommended_quantity).to be < usage_arr.max() + 1
            expect(recommended_quantity).to be > usage_arr.min() - 1
          end
        }
      end
    end

    context 'Recommendations details requests: Last 30 Days, 3 Year, No Upfront' do
      include_examples 'Usage details tests', 'Last30Days', 'ThreeYear', 'NoUpfront'
    end

    context 'Recommendations details requests: Last 7 Days, 1 Year, All Upfront' do
      include_examples 'Usage details tests', 'Last7Days', 'OneYear', 'AllUpfront'
    end

    # unskip or delete below tests on resolution of https://cloudhealthtech.atlassian.net/browse/ENG-50342
    xcontext 'Recommendations details requests: Last 60 Days, 3 Year, Partial Upfront' do
      include_examples 'Usage details tests', 'Last60Days', 'ThreeYear', 'PartialUpfront'
    end

    # unskip or delete below tests on resolution of https://cloudhealthtech.atlassian.net/browse/ENG-50340
    xcontext 'Recommendations details requests: Last 90 Days, 1 Year, Partial Upfront' do
      include_examples 'Usage details tests', 'Last90Days', 'OneYear', 'PartialUpfront'
    end
  end
end
