require 'spec_helper'

module ReservationRecommendationsFilterTest
  RSpec.shared_examples 'standard filter tests' do |scope, scope_level, scope_values|
    include_context 'request metadata'
    unfiltered_recommendations_array = []
    before(:each) {
      if unfiltered_recommendations_array.empty?
        if scope_values.nil?
          if USE_AZURE_SUBSCRIPTION_API
            all_available_subs = get_all_subscriptions()
            value_with_most_recs = get_value_with_most_quotes('Single', 'SUBSCRIPTION', all_available_subs)
            scope_values = value_with_most_recs
          else
            skip(SKIP_AZURE_SUBSCRIPTION_MSG)
          end
        end

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values}
        }
        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges].each { |recommendation|
          unfiltered_recommendations_array << recommendation
        }
        recommendation_presence_check(unfiltered_recommendations_array, 'AZURE')
      end
    }

    context 'Single filter tests' do
      it 'EQ with uppercase string (reservationOrderTerm)' do
        expected_filtered_quote_number = 0
        unfiltered_recommendations_array.each { |rec|
          if rec[:node][:savings][:reservationOrderTerm] == 'P3Y'
            expected_filtered_quote_number += 1
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: "reservationOrderTerm",
             filterType: "EQ",
             filterValues: ["p3y"]}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        filtered_recommendations =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(filtered_recommendations.length).to eq(expected_filtered_quote_number)
        # all returned quotes should have filtered term
        filtered_recommendations.each { |rec|
          expect(rec[:node][:savings][:reservationOrderTerm]).to eq('P3Y')
        }
        if filtered_recommendations.length > 0
          azure_recommendations_verification(scope_values, variables, json_body)
        end
      end

      it 'EQ with lowercase string (region)' do
        expected_filtered_quote_number = 0
        unfiltered_recommendations_array.each { |rec|
          if rec[:node][:resource][:region] == 'eastus'
            expected_filtered_quote_number += 1
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'region',
             filterType: 'EQ',
             filterValues: ['eastus']}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        filtered_recommendations =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(filtered_recommendations.length).to eq(expected_filtered_quote_number)
        filtered_recommendations.each { |rec|
          expect(rec[:node][:resource][:region]).to eq('eastus')
        }
        if filtered_recommendations.length > 0
          azure_recommendations_verification(scope_values, variables, json_body)
        end
      end

      it 'EQ case insensitivity (region: eastus to EASTUS)' do
        expected_filtered_quote_number = 0
        unfiltered_recommendations_array.each { |rec|
          if rec[:node][:resource][:region] == 'eastus'
            expected_filtered_quote_number += 1
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'region',
             filterType: 'EQ',
             filterValues: ['EASTUS']}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        filtered_recommendations =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(filtered_recommendations.length).to eq(expected_filtered_quote_number)
        filtered_recommendations.each { |rec|
          expect(rec[:node][:resource][:region]).to eq('eastus')
        }
        if filtered_recommendations.length > 0
          azure_recommendations_verification(scope_values, variables, json_body)
        end
      end

      it 'EQ with number (recommendedQuantity)' do
        recommended_quantity_value =
            unfiltered_recommendations_array[1][:node][:savings][:recommendedQuantity]
        expected_filtered_quote_number = 0
        unfiltered_recommendations_array.each { |rec|
          if rec[:node][:savings][:recommendedQuantity] == recommended_quantity_value
            expected_filtered_quote_number += 1
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'recommendedQuantity',
             filterType: 'EQ',
             filterValues: [recommended_quantity_value]}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        filtered_recommendations =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(filtered_recommendations.length).to eq(expected_filtered_quote_number)
        filtered_recommendations.each { |rec|
          expect(rec[:node][:savings][:recommendedQuantity]).to eq(recommended_quantity_value)
        }
        if filtered_recommendations.length > 0
          azure_recommendations_verification(scope_values, variables, json_body)
        end
      end

      it 'NE with string (lookBackPeriod)' do
        expected_filtered_quote_number = 0
        unfiltered_recommendations_array.each { |rec|
          if rec[:node][:savings][:lookBackPeriod] != 'Last7Days'
            expected_filtered_quote_number += 1
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'lookBackPeriod',
             filterType: 'NE',
             filterValues: ['Last7Days']}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        filtered_recommendations =
          json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(filtered_recommendations.length).to eq(expected_filtered_quote_number)
        filtered_recommendations.each { |rec|
          expect(['Last30Days', 'Last60Days']).to include(rec[:node][:savings][:lookBackPeriod])
        }
        if filtered_recommendations.length > 0
          azure_recommendations_verification(scope_values, variables, json_body)
        end
      end

      it 'NE with number (monthlySavings)' do
        monthly_savings_value =
            unfiltered_recommendations_array[0][:node][:savings][:monthlySavings]
        expected_filtered_quote_number = 0
        unfiltered_recommendations_array.each { |rec|
          if rec[:node][:savings][:monthlySavings] != monthly_savings_value
            expected_filtered_quote_number += 1
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'monthlySavings',
             filterType: 'NE',
             filterValues: [monthly_savings_value]}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        filtered_recommendations =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(filtered_recommendations.length).to eq(expected_filtered_quote_number)
        filtered_recommendations.each { |rec|
          expect(rec[:node][:savings][:monthlySavings]).to_not eq(monthly_savings_value)
        }
        if filtered_recommendations.length > 0
          azure_recommendations_verification(scope_values, variables, json_body)
        end
      end

      it 'GT (upfrontCost)' do
        upfront_cost_array = []
        upfront_cost_total = 0
        unfiltered_recommendations_array.each { |rec|
          upfront_cost_array << rec[:node][:savings][:upfrontCost]
          upfront_cost_total = upfront_cost_total + rec[:node][:savings][:upfrontCost]
        }
        mean_costs = upfront_cost_total / upfront_cost_array.length.to_f

        expected_filtered_quote_number = 0
        upfront_cost_array.each { |cost|
          if cost > mean_costs
            expected_filtered_quote_number += 1
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'upfrontCost',
             filterType: 'GT',
             filterValues: [mean_costs]}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        filtered_recommendations =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(filtered_recommendations.length).to eq(expected_filtered_quote_number)
        filtered_recommendations.each { |rec|
          expect(rec[:node][:savings][:upfrontCost]).to be > mean_costs
        }
        if filtered_recommendations.length > 0
          azure_recommendations_verification(scope_values, variables, json_body)
        end
      end

      it 'GTE (monthlyPayment)' do
        monthly_payment_array = []
        monthly_payment_total = 0
        unfiltered_recommendations_array.each { |rec|
          monthly_payment_array << rec[:node][:savings][:monthlyPayment]
          cost_with_no_ri_total = monthly_payment_total + rec[:node][:savings][:monthlyPayment]
        }
        array_length = monthly_payment_array.length
        mean_costs = monthly_payment_total / array_length.to_f

        expected_filtered_quote_number = 0
        monthly_payment_array.each { |cost|
          if cost >= mean_costs
            expected_filtered_quote_number += 1
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'monthlyPayment',
             filterType: 'GTE',
             filterValues: [mean_costs]}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        filtered_recommendations =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(filtered_recommendations.length).to eq(expected_filtered_quote_number)
        filtered_recommendations.each { |rec|
          expect(rec[:node][:savings][:monthlyPayment]).to be >= mean_costs
        }
        if filtered_recommendations.length > 0
          azure_recommendations_verification(scope_values, variables, json_body)
        end
      end

      it 'LT (savings)' do
        net_savings_array = []
        net_savings_total = 0

        unfiltered_recommendations_array.each { |rec|
          net_savings_array << rec[:node][:savings][:savings]
          net_savings_total = net_savings_total + rec[:node][:savings][:savings]
        }
        mean_savings = net_savings_total / net_savings_array.length.to_f

        expected_filtered_quote_number = 0
        net_savings_array.each { |cost|
          if cost < mean_savings
            expected_filtered_quote_number += 1
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'savings',
             filterType: 'LT',
             filterValues: [mean_savings]}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        filtered_recommendations =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(filtered_recommendations.length).to eq(expected_filtered_quote_number)
        filtered_recommendations.each { |rec|
          expect(rec[:node][:savings][:savings]).to be < mean_savings
        }
        if filtered_recommendations.length > 0
          azure_recommendations_verification(scope_values, variables, json_body)
        end
      end

      it 'LTE (annualSavings)' do
        annual_savings_array = []
        annual_savings_total = 0

        unfiltered_recommendations_array.each { |rec|
          annual_savings_array << rec[:node][:savings][:annualSavings]
          annual_savings_total = annual_savings_total + rec[:node][:savings][:annualSavings]
        }
        mean_savings = annual_savings_total / annual_savings_array.length.to_f

        expected_filtered_quote_number = 0
        annual_savings_array.each { |cost|
          if cost <= mean_savings
            expected_filtered_quote_number += 1
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'annualSavings',
             filterType: 'LTE',
             filterValues: [mean_savings]}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        filtered_recommendations =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(filtered_recommendations.length).to eq(expected_filtered_quote_number)
        filtered_recommendations.each { |rec|
          expect(rec[:node][:savings][:annualSavings]).to be <= mean_savings
        }
        if filtered_recommendations.length > 0
          azure_recommendations_verification(scope_values, variables, json_body)
        end
      end

      it 'MATCHES (region)' do
        expected_filtered_quote_number = 0
        unfiltered_recommendations_array.each { |rec|
          if rec[:node][:resource][:region].include?('east')
            expected_filtered_quote_number += 1
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'region',
             filterType: 'MATCHES',
             filterValues: ['east']}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        filtered_recommendations =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(filtered_recommendations.length).to eq(expected_filtered_quote_number)
        filtered_recommendations.each { |rec|
          expect(rec[:node][:resource][:region]).to include('east')
        }
        if filtered_recommendations.length > 0
          azure_recommendations_verification(scope_values, variables, json_body)
        end
      end

      it 'NOT_MATCHES (reservationOrderTerm)' do
        expected_filtered_quote_number = 0
        unfiltered_recommendations_array.each { |rec|
          if rec[:node][:savings][:reservationOrderTerm] == 'P3Y'
            expected_filtered_quote_number += 1
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'reservationOrderTerm',
             filterType: 'NOT_MATCHES',
             filterValues: ['1Y']}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        filtered_recommendations =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(filtered_recommendations.length).to eq(expected_filtered_quote_number)
        filtered_recommendations.each { |rec|
          expect( rec[:node][:savings][:reservationOrderTerm]).to eq('P3Y')
        }
        if filtered_recommendations.length > 0
          azure_recommendations_verification(scope_values, variables, json_body)
        end
      end

      it 'null (scopeName)' do
        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'scopeName',
             filterType: 'NULL',
             filterValues: []}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        # scopeName should be NULL for recommendations returned under Shared scope, not null under Single
        if scope == 'Shared'
          expect(json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges].length).to be >= 1
        else
          expect(json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges].length).to eq(0)
        end
      end

      # expect no null values
      it 'NOT_NULL (product)' do
        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'product',
             filterType: 'NOT_NULL',
             filterValues: ['Standard_B1s']}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        filtered_recommendations =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(filtered_recommendations.length).to eq(unfiltered_recommendations_array.length)
        if filtered_recommendations.length > 0
          azure_recommendations_verification(scope_values, variables, json_body)
        end
      end

      it 'EQ for recommendationScope Value (scopeValue)' do
        initial_variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
        }
        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            initial_variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        expected_filtered_quote_number = 0
        # get expected number of recommendations with filtered scope value
        json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges].each { |rec|
          if rec[:node][:scopeValue] == scope_values[0]
            expected_filtered_quote_number += 1
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'scopeValue',
             filterType: 'EQ',
             filterValues: [scope_values[0]]}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        filtered_recommendations =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(filtered_recommendations.length).to eq(expected_filtered_quote_number)
        filtered_recommendations.each { |rec|
          expect(rec[:node][:scopeValue]).to eq(scope_values[0])
        }
        if filtered_recommendations.length > 0
          azure_recommendations_verification(scope_values, variables, json_body)
        end
      end

      it 'MATCHES for recommendationScope Name (scopeName)' do
        if scope == 'Shared'
          skip('scopeName is null in shared scope')
        end
        scope_name_filter = unfiltered_recommendations_array[0][:node][:scopeName]

        initial_variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
        }
        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            initial_variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        expected_filtered_quote_number = 0
        # get expected number of recommendations with filtered scope name
        json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges].each { |rec|
          if rec[:node][:scopeName] == scope_name_filter
            expected_filtered_quote_number += 1
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'scopeName',
             filterType: 'MATCHES',
             filterValues: [scope_name_filter]}
          ]
        }
        # request with scopeName MATCHES filter
        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        filtered_recommendations =
          json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(filtered_recommendations.length).to eq(expected_filtered_quote_number)
        filtered_recommendations.each { |rec|
          expect(rec[:node][:scopeName]).to eq(scope_name_filter)
        }
        if filtered_recommendations.length > 0
          azure_recommendations_verification(scope_values, variables, json_body)
        end
      end

      it 'EQ filtered request with multiple valid values in filterValues array (location)' do
        expected_filtered_quote_number = 0
        unfiltered_recommendations_array.each { |rec|
          if rec[:node][:resource][:region] == 'eastus' ||
              rec[:node][:resource][:region] == 'westus2'
            expected_filtered_quote_number += 1
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'region',
             filterType: 'EQ',
             filterValues: ['eastus','westus2']}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        filtered_recommendations =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(filtered_recommendations.length).to eq(expected_filtered_quote_number)
        # all returned quotes should only have either location specified in filterValues array
        filtered_recommendations.each { |rec|
          location = rec[:node][:resource][:region]
          expect(['eastus', 'westus2']).to include(location)
          expected_location_returned = false
          if location == 'eastus' || location == 'westus2'
            expected_location_returned = true
          end
          expect(expected_location_returned).to eq(true),
              "unexpected location returned, expected eastus or westus2, but found #{location}"
          if filtered_recommendations.length > 0
            azure_recommendations_verification(scope_values, variables, json_body)
          end
        }
      end

      it 'NE filtered request with multiple valid values in filterValues array (lookBackPeriod)' do
        expected_filtered_quote_number = 0
        unfiltered_recommendations_array.each { |rec|
          if rec[:node][:savings][:lookBackPeriod] != 'Last7Days' &&
              rec[:node][:savings][:lookBackPeriod] != 'Last60Days'
            expected_filtered_quote_number += 1
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'lookBackPeriod',
             filterType: 'NE',
             filterValues: ['Last7Days', 'Last60Days']}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        filtered_recommendations =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(filtered_recommendations.length).to eq(expected_filtered_quote_number)
        filtered_recommendations.each { |rec|
          expect(rec[:node][:savings][:lookBackPeriod]).to eq('Last30Days')
        }
        if filtered_recommendations.length > 0
          azure_recommendations_verification(scope_values, variables, json_body)
        end
      end
    end

    context 'Multiple filter tests' do
      it '2 String filters: MATCH on 2 fields' do
        expected_filtered_quote_number = 0

        unfiltered_recommendations_array.each { |rec|
          if rec[:node][:resource][:region].include?('east') &&
              rec[:node][:savings][:reservationOrderTerm].include?('P3Y')
            expected_filtered_quote_number += 1
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'region',
             filterType: 'MATCHES',
             filterValues: ['east']},
            {field: 'reservationOrderTerm',
             filterType: 'MATCHES',
             filterValues: ['P3Y']}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        filtered_recommendations =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(filtered_recommendations.length).to eq(expected_filtered_quote_number)
        filtered_recommendations.each { |rec|
          expect(rec[:node][:resource][:region]).to include('east')
          expect(rec[:node][:savings][:reservationOrderTerm]).to include('P3Y')
        }
        if filtered_recommendations.length > 0
          azure_recommendations_verification(scope_values, variables, json_body)
        end
      end

      it '2 String filters: EQ on 2 fields' do
        expected_filtered_quote_number = 0
        unfiltered_recommendations_array.each { |rec|
          if rec[:node][:savings][:lookBackPeriod] == 'Last30Days' &&
              rec[:node][:savings][:reservationOrderTerm] == 'P1Y'
            expected_filtered_quote_number += 1
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'lookBackPeriod',
             filterType: 'EQ',
             filterValues: ['Last30Days']},
            {field: 'reservationOrderTerm',
             filterType: 'EQ',
             filterValues: ['P1Y']}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end

        filtered_recommendations =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(filtered_recommendations.length).to eq(expected_filtered_quote_number)
        filtered_recommendations.each { |rec|
          expect(rec[:node][:savings][:lookBackPeriod]).to eq('Last30Days')
          expect(rec[:node][:savings][:reservationOrderTerm]).to eq('P1Y')
        }
        if filtered_recommendations.length > 0
          azure_recommendations_verification(scope_values, variables, json_body)
        end
      end

      it '4 string filters: MATCH and EQ FILTERS' do
        initial_variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
        }
        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            initial_variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end

        expected_quotes_with_filter= 0
        unfiltered_recommendations_array.each { |rec|
          if rec[:node][:savings][:reservationOrderTerm] == 'P1Y' &&
              rec[:node][:resource][:region].include?('east') &&
              rec[:node][:savings][:lookBackPeriod] == 'Last30Days' &&
              rec[:node][:resource][:resourceType].include?('Virtual')
            expected_quotes_with_filter += 1
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'reservationOrderTerm',
             filterType: 'EQ',
             filterValues: ['P1Y']},
            {field: 'region',
             filterType: 'MATCHES',
             filterValues: ['east']},
            {field: 'lookBackPeriod',
             filterType: 'EQ',
             filterValues: ['Last30Days']},
            {"field": 'resourceType',
             "filterType": 'MATCHES',
             "filterValues": ['virtual']}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        filtered_recommendations =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(filtered_recommendations.length).to eq(expected_quotes_with_filter)
        filtered_recommendations.each { |rec|
          expect(rec[:node][:resource][:region]).to include('east')
          expect(rec[:node][:savings][:reservationOrderTerm]).to eq('P1Y')
          expect(rec[:node][:savings][:lookBackPeriod]).to eq('Last30Days')
          expect(rec[:node][:resource][:resourceType]).to include('Virtual')
        }
        if filtered_recommendations.length > 0
          azure_recommendations_verification(scope_values, variables, json_body)
        end
      end

      it '2 number filters: GT and LT on same field (upfrontCost value range)' do
        upfront_cost_array = []
        upfront_cost_total = 0
        unfiltered_recommendations_array.each { |rec|
          upfront_cost_array << rec[:node][:savings][:upfrontCost]
          upfront_cost_total = upfront_cost_total + rec[:node][:savings][:upfrontCost]
        }
        array_length = upfront_cost_array.length
        mean_costs = upfront_cost_total / array_length.to_f
        # want upper limit to be less than max, to ensure LT filter works
        lower_limit = (mean_costs - (mean_costs / 2))
        upper_limit = (mean_costs + (mean_costs / 2))

        cost_range = 0
        upfront_cost_array.each { |cost|
          if cost >= lower_limit && cost < upper_limit
            cost_range += 1
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'upfrontCost',
             filterType: 'GT',
             filterValues: [lower_limit]},
            {field: 'upfrontCost',
             filterType: 'LT',
             filterValues: upper_limit}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        filtered_recommendations =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(filtered_recommendations.length).to eq(cost_range)
        filtered_recommendations.each { |rec|
          expect(rec[:node][:savings][:upfrontCost]).to be >= lower_limit
          expect(rec[:node][:savings][:upfrontCost]).to be < upper_limit
        }
        if filtered_recommendations.length > 0
          azure_recommendations_verification(scope_values, variables, json_body)
        end
      end

      it '2 number filters: GT and LT on different number fields (annualSavings / monthlySavings)' do
        annual_savings_total = 0
        monthly_savings_total = 0
        unfiltered_recommendations_array.each { |rec|
          annual_savings_total = annual_savings_total + rec[:node][:savings][:annualSavings]
          monthly_savings_total = monthly_savings_total + rec[:node][:savings][:monthlySavings]
        }
        mean_annual_savings = annual_savings_total / unfiltered_recommendations_array.length.to_f
        mean_monthly_savings = monthly_savings_total / unfiltered_recommendations_array.length.to_f
        annual_savings_filter = (mean_annual_savings / 2)
        monthly_savings_filter = (mean_monthly_savings * 2)

        expected_quotes_with_filter = 0
        unfiltered_recommendations_array.each { |rec|
          if rec[:node][:savings][:annualSavings] > annual_savings_filter &&
              rec[:node][:savings][:monthlySavings]< monthly_savings_filter
            expected_quotes_with_filter += 1
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'annualSavings',
             filterType: 'GT',
             filterValues: [annual_savings_filter]},
            {field: 'monthlySavings',
             filterType: 'LT',
             filterValues: [monthly_savings_filter]}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        filtered_recommendations =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(filtered_recommendations.length).to eq(expected_quotes_with_filter)
        filtered_recommendations.each { |rec|
          expect(rec[:node][:savings][:annualSavings] ).to be > annual_savings_filter
          expect(rec[:node][:savings][:monthlySavings] ).to be < monthly_savings_filter
        }
        if filtered_recommendations.length > 0
          azure_recommendations_verification(scope_values, variables, json_body)
        end
      end

      it '2 mixed number/string filters: EQ with GTE on string (reservationOrderTerm) and number (upfrontCost) fields' do
        upfront_cost_total = 0
        expected_quotes_with_filter= 0
        unfiltered_recommendations_array.each { |rec|
          upfront_cost_total = upfront_cost_total + rec[:node][:savings][:upfrontCost]
        }

        array_length = unfiltered_recommendations_array.length
        mean_costs = upfront_cost_total / array_length.to_f

        unfiltered_recommendations_array.each { |rec|
          if rec[:node][:savings][:reservationOrderTerm] == 'P3Y' &&
              rec[:node][:savings][:upfrontCost] >= mean_costs
            expected_quotes_with_filter += 1
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'reservationOrderTerm',
             filterType: 'EQ',
             filterValues: ['P3Y']},
            {field: 'upfrontCost',
             filterType: 'GTE',
             filterValues: [mean_costs]}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        filtered_recommendations =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(filtered_recommendations.length).to eq(expected_quotes_with_filter)
        filtered_recommendations.each { |rec|
          expect(rec[:node][:savings][:upfrontCost]).to be >= mean_costs
          expect(rec[:node][:savings][:reservationOrderTerm]).to eq('P3Y')
        }
        if filtered_recommendations.length > 0
          azure_recommendations_verification(scope_values, variables, json_body)
        end
      end

      it '2 mixed number/string filters: NE with GT on string (reservationOrderTerm) and number (upfrontCost) fields' do
        upfront_cost_total = 0
        expected_quotes_with_filter= 0
        unfiltered_recommendations_array.each { |rec|
          upfront_cost_total = upfront_cost_total + rec[:node][:savings][:upfrontCost]
        }
        mean_costs = upfront_cost_total / unfiltered_recommendations_array.length.to_f

        unfiltered_recommendations_array.each { |rec|
          if rec[:node][:savings][:reservationOrderTerm] == 'P3Y' &&
              rec[:node][:savings][:upfrontCost] > mean_costs
            expected_quotes_with_filter += 1
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'reservationOrderTerm',
             filterType: 'NE',
             filterValues: ['P1Y']},
            {field: 'upfrontCost',
             filterType: 'GT',
             filterValues: [mean_costs]}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        filtered_recommendations =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(filtered_recommendations.length).to eq(expected_quotes_with_filter)
        filtered_recommendations.each { |rec|
          expect(rec[:node][:savings][:upfrontCost]).to be > mean_costs
          expect(rec[:node][:savings][:reservationOrderTerm]).to eq('P3Y')
        }
        if filtered_recommendations.length > 0
          azure_recommendations_verification(scope_values, variables, json_body)
        end
      end

      it '2 filters:  EQ on uppercase string (term) with EQ on lowercase string (location)' do
        expected_quotes_with_filter= 0
        unfiltered_recommendations_array.each { |rec|
          if rec[:node][:savings][:reservationOrderTerm] == 'P1Y' &&
              rec[:node][:resource][:region] == 'eastus'
            expected_quotes_with_filter += 1
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'reservationOrderTerm',
             filterType: 'EQ',
             filterValues: ['P1Y']},
            {field: 'region',
             filterType: 'EQ',
             filterValues: ['eastus']}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        filtered_recommendations =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(filtered_recommendations.length).to eq(expected_quotes_with_filter)
        filtered_recommendations.each { |rec|
          expect(rec[:node][:resource][:region]).to eq('eastus')
          expect(rec[:node][:savings][:reservationOrderTerm]).to eq('P1Y')
        }
        if filtered_recommendations.length > 0
          azure_recommendations_verification(scope_values, variables, json_body)
        end
      end

      it '4 filters: EQ (location) with NE (term) with GTE (costWithNoReservedInstances) with NOT_NULL' do
        upfront_cost_total = 0
        expected_quotes_with_filter= 0
        unfiltered_recommendations_array.each { |rec|
          upfront_cost_total = upfront_cost_total + rec[:node][:savings][:upfrontCost]
        }
        mean_costs = upfront_cost_total / unfiltered_recommendations_array.length.to_f
        upfront_cost_filter = mean_costs / 2

        unfiltered_recommendations_array.each { |rec|
          if rec[:node][:savings][:reservationOrderTerm] == 'P3Y' &&
              rec[:node][:savings][:upfrontCost] >= upfront_cost_filter &&
              rec[:node][:resource][:region] == 'eastus'
            expected_quotes_with_filter += 1
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'reservationOrderTerm',
             filterType: 'NE',
             filterValues: ['P1Y']},
            {field: 'region',
             filterType: 'EQ',
             filterValues: ['eastus']},
            {field: 'upfrontCost',
             filterType: 'GTE',
             filterValues: [upfront_cost_filter]},
            {field: 'resourceType',
             filterType: 'NOT_NULL',
             filterValues: ['random']}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        filtered_recommendations =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(filtered_recommendations.length).to eq(expected_quotes_with_filter)
        filtered_recommendations.each { |rec|
          expect(rec[:node][:savings][:upfrontCost]).to be >= upfront_cost_filter
          expect(rec[:node][:savings][:reservationOrderTerm]).to eq('P3Y')
          expect(rec[:node][:resource][:region]).to eq('eastus')
        }
        if filtered_recommendations.length > 0
          azure_recommendations_verification(scope_values, variables, json_body)
        end
      end

      it '4 filters: EQ (region) with MATCHES (reservationOrderTerm) with LT (upfrontCost) with NE (lookBackPeriod)' do
        upfront_cost_total = 0
        expected_quotes_with_filter= 0
        unfiltered_recommendations_array.each { |rec|
          upfront_cost_total = upfront_cost_total + rec[:node][:savings][:upfrontCost]
        }
        mean_costs = upfront_cost_total / unfiltered_recommendations_array.length.to_f
        upfront_cost_filter = mean_costs * 1.5

        unfiltered_recommendations_array.each { |rec|
          if rec[:node][:savings][:reservationOrderTerm] == 'P3Y' &&
              rec[:node][:savings][:upfrontCost] < upfront_cost_filter &&
              rec[:node][:resource][:region] == 'eastus' &&
              rec[:node][:savings][:lookBackPeriod] != 'Last30Days'
            expected_quotes_with_filter += 1
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'reservationOrderTerm',
             filterType: 'MATCHES',
             filterValues: ['3Y']},
            {field: 'region',
             filterType: 'EQ',
             filterValues: ['eastus']},
            {field: 'upfrontCost',
             filterType: 'LT',
             filterValues: [upfront_cost_filter]},
            {field: 'lookBackPeriod',
             filterType: 'NE',
             filterValues: ['Last30Days']}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        filtered_recommendations =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(filtered_recommendations.length).to eq(expected_quotes_with_filter)
        filtered_recommendations.each { |rec|
          expect(rec[:node][:savings][:upfrontCost]).to be < upfront_cost_filter
          expect(rec[:node][:savings][:reservationOrderTerm]).to eq('P3Y')
          expect(rec[:node][:resource][:region]).to eq('eastus')
          expect(['Last7Days', 'Last60Days']).to include(rec[:node][:savings][:lookBackPeriod])
        }
        if filtered_recommendations.length > 0
          azure_recommendations_verification(scope_values, variables, json_body)
        end
      end
    end
  end

  describe 'ReservationRecommendations azureRIRecDetailsInfo: filter tests' do
    include_context 'request metadata'

    context 'Single scope, SUBSCRIPTION scope level, single subscription' do
      include_examples 'standard filter tests', 'Single', 'SUBSCRIPTION'
    end

    context 'Single scope, ENROLLMENT scope level cases (All Subscriptions/Default)' do
      include_examples 'standard filter tests', 'Single', 'ENROLLMENT', [SHARED_SCOPE_IDS[:enrollment]]
    end

    context 'Shared scope, ENROLLMENT scope level cases' do
      include_examples 'standard filter tests', 'Shared', 'ENROLLMENT', [SHARED_SCOPE_IDS[:enrollment]]
    end

    # unskip or delete below tests on resolution of https://cloudhealthtech.atlassian.net/browse/ENG-47430
    xcontext 'Shared scope, BILLING_PROFILE scope level cases' do
      include_examples 'standard filter tests', 'Shared', 'BILLING_PROFILE', [SHARED_SCOPE_IDS[:billing_profile]]
    end

  end

  describe 'negative cases' do
    include_context 'request metadata'
    # TODO: enable test on resolution of ENG-37841
    it 'Invalid field value in filterRules' do
      variables = {
        recommendationScope: {
          scope: 'Shared',
          scopeLevel: 'ENROLLMENT',
          scopeValues: [SHARED_SCOPE_IDS[:enrollment]]},
        filterRules: [
          {field: 'RESERVATIONORDERTERM',
           filterType: 'EQ',
           filterValues: ['p3y']}
        ]
      }

      wrap_request do
        graphql url(service: service),
          format_request(
            :azureRIRecDetailsInfo,
            service: service
          ),
          variables, headers
        expect_status(200)
        expect(json_body[:errors][0][:extensions][:classification]).to eq("DataFetchingException")
        expect(json_body[:data]).to be_nil
      end
    end

    it 'Invalid filterType value in filterRules' do
      variables = {
        recommendationScope: {
          scope: 'Shared',
          scopeLevel: 'ENROLLMENT',
          scopeValues: [SHARED_SCOPE_IDS[:enrollment]]},
        filterRules: [
          {field: 'reservationOrderTerm',
           filterType: 'EQUALS',
           filterValues: ['p3y']}
        ]
      }

      wrap_request do
        graphql url(service: service),
          format_request(
            :azureRIRecDetailsInfo,
            service: service
          ),
          variables, headers
        expect_status(200)
        expect(json_body[:errors][0][:extensions][:classification]).to eq("ValidationError")
        expect(json_body[:data]).to be_nil
      end
    end
    
    it 'Invalid filterValues value in filterRules' do
      variables = {
        recommendationScope: {
          scope: 'Single',
          scopeLevel: 'ENROLLMENT',
          scopeValues: [SHARED_SCOPE_IDS[:enrollment]]},
        filterRules: [
          {field: 'scopeName',
           filterType: 'EQ',
           filterValues: ['Last60Days']}
        ]
      }

      wrap_request do
        graphql url(service: service),
          format_request(
            :azureRIRecDetailsInfo,
            service: service
          ),
          variables, headers
        expect_success_graphql
        check(:azureRIRecDetailsInfo, service: service)
      end
      recommendations =
          json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
      expect(recommendations.length).to eq(0)
    end

    it 'Valid and invalid filter values in same request' do
      expected_filtered_quote_number = 0

      variables = {
        recommendationScope: {
          scope: 'Shared',
          scopeLevel: 'ENROLLMENT',
          scopeValues: [SHARED_SCOPE_IDS[:enrollment]]}
      }
      wrap_request do
        graphql url(service: service),
          format_request(
              :azureRIRecDetailsInfo,
              service: service
          ),
          variables, headers
        expect_success_graphql
        check(:azureRIRecDetailsInfo, service: service)
      end
      json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges].each { |rec|
        if rec[:node][:savings][:reservationOrderTerm] == 'P3Y'
          expected_filtered_quote_number += 1
        end
      }

      variables = {
        recommendationScope: {
          scope: 'Shared',
          scopeLevel: 'ENROLLMENT',
          scopeValues: [SHARED_SCOPE_IDS[:enrollment]]},
        filterRules: [
          {field: 'reservationOrderTerm',
           filterType: 'EQ',
           filterValues: ['p2y', 'p3y']}
        ]
      }

      wrap_request do
        graphql url(service: service),
          format_request(
            :azureRIRecDetailsInfo,
            service: service
          ),
          variables, headers
        expect_success_graphql
        check(:azureRIRecDetailsInfo, service: service)
      end
      filtered_recommendations =
          json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
      # only reservations for valid filter value should be returned
      expect(filtered_recommendations.length).to eq(expected_filtered_quote_number)
      filtered_recommendations.each { |rec|
        expect(rec[:node][:savings][:reservationOrderTerm]).to eq('P3Y')
      }
    end
  end
end
