# frozen_string_literal: true

require 'cht-chaphql'
require 'cht_apitest_local_auth'
require 'cht_apitest_local_auth/service'

Dir[File.join(__dir__, '../lib/*', '*.rb')].sort.each { |file| require file }
Dir[File.join(__dir__, 'helpers/*.rb')].sort.each { |f| require f }

USER_DATA = YAML.load_file(File.join(__dir__, '../lib/helpers/data.yml'))

RSpec.configure do |c|
  c.include Chaphql
  c.include Chaphql::GraphQLHelpers
  c.include Chaphql::AnalyzerHelpers
  c.include Chaphql::StringManipulation
  c.include Chaphql::ServiceHelper
  c.include Chaphql::IntrospectConflict
  c.include ChtApitestLocalAuth
  c.include ChtApitestAuthCommon
  c.include Spaceborne
  include ChtApitestReservationRecommendations
  include ReservationRecommendationsHelpers
  c.filter_run_excluding(save_schema: true)
  c.filter_run_excluding(staging_env: true) unless ENV['TEST_ENV'] == 'staging'
  #c.filter_run_excluding(not_prod: true) if %{canary prod}.include?(getenv)
  c.filter_run_excluding(not_via_proxy: true) if ENV['EKS_ENV'] == 'external'
  c.shared_context_metadata_behavior = :apply_to_host_groups
end

RSpec.shared_context 'request metadata', shared_context: :metadata do
  let(:via_public) { %w[prod canary preprod staging].include?(getenv) }
  let(:service) { ChtApitestReservationRecommendations::ReservationRecommendationsService.new }
  let(:data_service) { DataConnectConfigService.new(via_public: via_public) }
  let(:auth_service) { LocalAuthService.new(via_public: via_public) }
  let(:sp_scheduler_service) { SpAllocationSchedulerService.new }
  let(:tokens) { login_api(ENV['API_KEY'], service: auth_service) }
  let(:headers) do
    apitest_headers(service: auth_service,
                    with_auth: true,
                    access_token: tokens[:access_token])
  end

  TEST_DATA_ENV = if ENV['TEST_ENV'] == 'preprod'
                    "preprod"
                  elsif ENV['TEST_ENV'] == 'staging'
                    "staging"
                  else
                    "default"
                  end

  let(:customerId) { USER_DATA[TEST_DATA_ENV]['customerId'].to_s }
  let(:dbg_headers) { headers.merge(use_curl: 'debug') }
  let(:get_new_tokens) do
    headers = apitest_headers(service: auth_service,
                              with_auth: true,
                              refresh: tokens[:refresh_token])
    dbg_headers = headers.merge(use_curl: 'debug')
  end
end

RSpec.configure do |c|
  c.include_context 'request metadata', include_shared: true
end

def get_azure_scope_values_array(recommendation_array)
  scope_values_array = []
  recommendation_array.each { |recommendation|
    scope_values_array << recommendation[:node][:scopeValue]
  }
  scope_values_array
end

# method to find the azure_id scopeValue with the most quotes associated with it
# we use this to find the optimum scopeValue to use for testing
def get_value_with_most_quotes(scope, scope_level, scope_values)
  variables = {
    recommendationScope: {
      scope: scope,
      scopeLevel: scope_level,
      scopeValues: scope_values
    }
  }

  wrap_request do
    graphql url(service: service),
      format_request(
        :azureRIRecDetailsInfo,
        service: service
      ),
      variables, headers
    expect_success_graphql
    check(:azureRIRecDetailsInfo, service: service)
  end
  recommendation_array =
    json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]

  all_scope_values_array = get_azure_scope_values_array(recommendation_array)
  value_frequency = Hash.new(0)
  all_scope_values_array.each{|key| value_frequency[key] += 1}
  highest_num = value_frequency.values.max
  return value_frequency.key(highest_num)
end

def get_all_subscriptions ()
  azure_subscription_array = []
  wrap_request do
    variables = {
    }
    graphql url(service: data_service),
      format_request(
        :azureSubscriptions,
        service: data_service
      ),
      variables, headers
    expect_success_graphql
    check(:azureSubscriptions, service: data_service)
  end
  json_body[:data][:azureSubscriptions][:edges].each {|sub|
    azure_subscription_array << sub[:node][:azureId]
  }
  return azure_subscription_array
end

AZURE_RESERVATION_TERMS = ['P1Y', 'P3Y']
AZURE_LOOKBACK_PERIODS = ['Last7Days', 'Last30Days', 'Last60Days']
AZURE_RESERVABLE_SERVICE_TYPES = ['VirtualMachines', 'SQLDatabases', 'PostgreSQL',
                                 'ManagedDisk', 'MySQL', 'RedHat', 'MariaDB',
                                 'RedisCache', 'CosmosDB', 'SqlDataWarehouse', 'SUSELinux',
                                 'AppService', 'BlockBlob', 'AzureDataExplorer', 'VMwareCloudSimple']

def azure_recommendations_verification(scope_values_in_request, variables, api_response)
  all_subs_request = false
  if variables[:recommendationScope][:scope] == 'Single' &&
      (variables[:recommendationScope][:scopeLevel] == 'ENROLLMENT' ||
          variables[:recommendationScope][:scopeLevel] == 'BILLING_PROFILE')
    all_subs_request = true
  end

  projected_monthly_savings = api_response[:data][:azureRIRecDetailsInfo][:projectedMonthlySavings]
  projected_annual_savings = api_response[:data][:azureRIRecDetailsInfo][:projectedAnnualSavings]
  upfront_costs = json_body[:data][:azureRIRecDetailsInfo][:totalUpfrontCost]
  expect(projected_monthly_savings).to be > 0
  # allowing for a 6% delta due to annual savings being calculated via net_savings/hours_in_lookback_period
  # rather than calculated from existing monthly savings by 12
  expected_annual_savings = (projected_monthly_savings * 12).round(2)
  delta = (expected_annual_savings) * 0.06
  expect(projected_annual_savings.round(2)).to be_within(delta).of(expected_annual_savings)

  total_savings_opportunities = api_response[:data][:azureRIRecDetailsInfo][:totalSavingsOpportunities]
  # this would also fail if there happened to be no reservations in the account
  # but we would still want to know that, to ensure all tests have valid data to run on
  expect(total_savings_opportunities).to be > 0

  total_count  = api_response[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:totalCount]
  expect(total_count).to be > 0
  recommendation_array =
      api_response[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]

  if variables[:first].nil? && variables[:last].nil?
    expect(recommendation_array.length).to eq(total_count)
  else
    expect(recommendation_array.length).to be <= total_count
  end
  total_upfront_costs = 0.0
  total_monthly_net_savings = 0.0
  total_annual_net_savings = 0.0
  expected_total_opportunities = 0.0
  recommendation_array.each { |recommendation|
    if variables[:recommendationScope][:scope] == 'Single'
      expect(recommendation[:node][:resource][:appliedScopes].length).to eq(0)
    end
    expect(recommendation[:node][:savings][:recommendedQuantity]).to be > 0
    expected_total_opportunities = expected_total_opportunities + recommendation[:node][:savings][:recommendedQuantity]
    expect(recommendation[:node][:scope]).to eq(variables[:recommendationScope][:scope])
    if all_subs_request == false
      expect(recommendation[:node][:scopeLevel]).to eq(variables[:recommendationScope][:scopeLevel])
      expect(scope_values_in_request).to include(recommendation[:node][:scopeValue])
    end
    if variables[:recommendationScope][:scope] == 'Shared'
      expect(recommendation[:node][:scopeName]).to eq(nil)
    else
      expect(recommendation[:node][:scopeName]).to_not eq(nil)
      expect(recommendation[:node][:scopeName]).to be_a(String)
    end
    expect(AZURE_RESERVATION_TERMS).to include(recommendation[:node][:savings][:reservationOrderTerm])
    expect(AZURE_LOOKBACK_PERIODS).to include(recommendation[:node][:savings][:lookBackPeriod])
    expect(AZURE_RESERVABLE_SERVICE_TYPES).to include(recommendation[:node][:resource][:resourceType])

    expect(recommendation[:node][:savings][:upfrontCost]).to be > 0
    expect(recommendation[:node][:savings][:savings]).to be > 0
    expect(recommendation[:node][:savings][:monthlySavings]).to be > 0
    expect(recommendation[:node][:savings][:annualSavings]).to be > 0
    net_savings = recommendation[:node][:savings][:savings]
    total_upfront_costs += recommendation[:node][:savings][:upfrontCost]
    term = recommendation[:node][:savings][:reservationOrderTerm] == 'P1Y' ? 1 : 3
    expect(recommendation[:node][:savings][:monthlySavings].round(2)).
        to eq((net_savings / (term * 12)).round(2))
    expect(recommendation[:node][:savings][:annualSavings].round(2)).
        to eq((net_savings / term).round(2))
    total_monthly_net_savings += (net_savings / (term * 12))
    total_annual_net_savings += (net_savings / term)

  }
  # summary values won't match when pagination is specified
  if variables[:first].nil? && variables[:last].nil?
    expect(upfront_costs.round(2)).to eq(total_upfront_costs.round(2))
    expect(projected_monthly_savings.round(2)).to eq(total_monthly_net_savings.round(2))
    expect(projected_annual_savings.round(2)).to eq(total_annual_net_savings.round(2))
    expect(projected_annual_savings.round(2)).to eq(total_annual_net_savings.round(2))
    expect(total_savings_opportunities.round(2)).to eq(expected_total_opportunities.round(2))
  end
end

def aws_recommendations_verification(api_response, period, term, services, payment_type)
  reservation_recommendations = api_response[:reservationRecommendations][:edges]
  total_recommendations = api_response[:reservationRecommendations][:totalCount]
  expect(reservation_recommendations.length).to be > 0
  expect(reservation_recommendations.length).to eq(total_recommendations)
  reservation_recommendations.each { |recommendation|
    expect(recommendation[:node][:savingsProperties][:recommendedQuantity]).to be > 0
    # TODO: uncomment on resolution of ENG-46946
    # expect(recommendation[:node][:savingsProperties][:newCoverage]).to be > recommendation[:node][:savingsProperties][:currentCoverage]
    if payment_type == 'AllUpfront'
      if payment_type == recommendation[:node][:savingsProperties][:paymentType]
        expect(recommendation[:node][:savingsProperties][:monthlyPayment]).to eq(0)
      else
        expect(recommendation[:node][:resourceProperties][:serviceType]).to eq('DynamoDB')
        expect(recommendation[:node][:savingsProperties][:monthlyPayment]).to be > 0
      end
    else
      expect(recommendation[:node][:savingsProperties][:monthlyPayment]).to be > 0
    end
    expect(recommendation[:node][:savingsProperties][:monthlySavings]).to be > 0
    if payment_type == 'NoUpfront'
      expect(recommendation[:node][:savingsProperties][:upfrontCost]).to eq(0)
    else
      expect(recommendation[:node][:savingsProperties][:upfrontCost]).to be > 0
    end
    expect(recommendation[:node][:savingsProperties][:annualSavings]).to be_within(1).of(
        recommendation[:node][:savingsProperties][:monthlySavings] * 12)
    expect(recommendation[:node][:savingsProperties][:analysisLookBackPeriod]).to eq(period)
    net_savings = recommendation[:node][:savingsProperties][:savings]
    expect(net_savings).to be_within(1).of(recommendation[:node][:savingsProperties][:annualSavings] * term)
    expect(net_savings).to be_within(1).of(recommendation[:node][:savingsProperties][:monthlySavings] * (term * 12))

    if services.length > 0
      expect(services).to include(recommendation[:node][:resourceProperties][:serviceType])
    end
  }
end

SHARED_SCOPE_IDS = {
  :enrollment =>  '6366418',
  :billing_profile => '95e281a2-766f-46de-ae3b-3d9a4b35135c:40e01c51-8cb4-42a6-99bb-8f14a254a885_2018-09-30|5yzw-w4e6-bg7-pgb'
}

EA_SINGLE_SUBSCRIPTION_IDS = ['f2a56d89-4864-4801-9276-359b4fc4b6e5',
                              '700f3a5c-8c56-44b9-aa1d-fcfb1493aa4b',
                              '151f9055-7a93-4bbb-8de3-e8af62caa854',
                              'c0ef2db4-fada-4539-a6de-8bedd2a0743f',
                              'e8c052cc-1a8e-4beb-a443-ba9773a705d4',
                              '5baee7fd-a650-4d13-ade2-5303c618eb2e',
                              '5e1455be-dd28-4290-b33b-141fd1bdcd3e']

MCA_SINGLE_SUBSCRIPTION_IDS = ['38553983-b252-47ec-879d-5299151a8434']

PAYG_SINGLE_SUBSCRIPTION_IDS = ['cd5921a6-6f77-4505-81d7-efe42bfb6023',
                                '184ec6fc-fd37-4f88-9e74-9383e348a7fb',
                                '9015bab3-0e11-4fed-a5ac-2fff4698a0a0',
                                'e7092a41-599b-47a8-b1f6-54f1c7c957c7',
                                'd687b1a3-9b78-43b1-a17b-7de297fd1fce',
                                '8aa70cc7-bf51-4e8c-baa0-368cb78b3c0c',
                                '5228b508-61a8-401d-be65-18b819fb0ef4',
                                '1e5be4be-bdc4-4c43-b0cc-bdbb05c91652']

# flag to allow us to enable/disable tests that use azureSubscriptions api
USE_AZURE_SUBSCRIPTION_API = ENV['OVERRIDE_ENDPOINT'] ? false : true
SKIP_AZURE_SUBSCRIPTION_MSG = 'skipping test due to not being able to use azureSubscriptions api'

def get_update_job_id(period, payment_type, term, services)
    wrap_request do
    variables = {
      analysisInput: {
        awsAnalysisInput: {
          applicationScope: 'Regional',
          billingAccountId: "crn:#{customerId}:aws-account/#{TEST_DATA_ENV == 'preprod' ? REDSHIRT_PREPROD[:owner_id] : REDSHIRT[:owner_id]}",
          linkedAccounts: [
            "crn:#{customerId}:aws-account/#{TEST_DATA_ENV == 'preprod' ? REDSHIRT_PREPROD[:owner_id] : REDSHIRT[:owner_id]}",
            "crn:#{customerId}:aws-account/#{TEST_DATA_ENV == 'preprod' ? REDSHIRT_2_PREPROD[:owner_id] : REDSHIRT_2[:owner_id]}",
            "crn:#{customerId}:aws-account/#{TEST_DATA_ENV == 'preprod' ? REDSHIRT_3_PREPROD[:owner_id] : REDSHIRT_3[:owner_id]}"
          ],
          lookBackPeriod: {
            period: period
          },
          paymentTypes: [
            payment_type
          ],
          recommendationOption: {
            isDefault: true
          },
          regions: [],
          services: services,
          term: term
        }
      },
      cloud: 'AWS'
    }
    graphql url(service: service),
      format_request(
        :updateReservationRecommendations,
        service: service
      ),
      variables, headers
    expect_success_graphql
    check(:updateReservationRecommendations, service: service)
  end
  job_id = json_body[:data][:updateReservationRecommendations][:id]
  aws_job_completion_check(job_id)
  return job_id
end

def aws_job_completion_check(job_id)
  # the time to allow for a quote to get to COMPLETED status
  # is a combination of the wait and retry values below
  # Currently, the test allows 3 minutes for quote creation to COMPLETE
  wait_time_increment = 10
  retry_attempts = 19
  time_for_complete = wait_time_increment * (retry_attempts - 1)
  puts "getting intial status for job #{job_id}"
  wrap_request do
    variables = {}
    graphql url(service: service),
      format_request(
        :node,
        service: service,
        crn: job_id,
        type: 'AwsReservationRecommendationJob',
        fields: 'id,
          status,
          sourceArtifactStatus{
              asOfDate
             needRefresh
          }'
      ),
      variables, headers
  end

  quote_status = json_body[:data][:node][:status]
  if quote_status == 'FAILED'
    raise Exception, "recommendation creation job failed: #{json_body}"
  end

  attempts = 0
  wait_time = wait_time_increment
  while quote_status != 'COMPLETED' && attempts < retry_attempts
    sleep(wait_time_increment)
    wait_time += wait_time_increment
    puts "waiting #{wait_time} seconds for create quote completion (job id: #{job_id}, status: #{quote_status})"
    wrap_request do
      variables = {}
      graphql url(service: service),
        format_request(
          :node,
          service: service,
          crn: job_id,
          type: 'AwsReservationRecommendationJob',
          fields: 'id,
            status,
            sourceArtifactStatus{
                  asOfDate
                 needRefresh
            }'
        ),
        variables, headers
    end
    quote_status = json_body[:data][:node][:status]
    # fail fast if quote status goes to FAILED
    if quote_status == 'FAILED'
      raise Exception, "recommendation creation job failed: #{json_body}"
    end
    attempts += 1
  end
  if quote_status != 'COMPLETED'
    raise Exception, "Quote #{job_id} did not COMPLETE in #{time_for_complete} seconds, status: #{quote_status}"
  end
end

def recommendation_presence_check(rec_array, cloud)
  if rec_array.length == 0
    raise Exception, "No #{cloud} recommendations available in #{getenv}"
  end

end

REDSHIRT = {
  :aws_billing_id =>  3367254377911,
  :customer_id => 2017,
  :organization_id => 893353197629,
  :owner_id => '844584796563',
  :name => 'Redshirt'
}

REDSHIRT_2 = {
  :aws_billing_id =>  '3367254362284',
  :owner_id => '081309885503',
  :name => 'Redshirt 2'
}

REDSHIRT_3 = {
  :aws_billing_id =>  '3367254362285',
  :owner_id => '781971168076',
  :name => 'Redshirt 3'
}

# adding values for preprod

REDSHIRT_PREPROD = {
  :aws_billing_id =>  3,
  :customer_id => 57534,
  :organization_id => 5841155524188,
  :owner_id => '844584796563',
  :name => 'Redshirt'
}

REDSHIRT_2_PREPROD = {
  :aws_billing_id =>  '6',
  :owner_id => '081309885503',
  :name => 'Redshirt 2'
}

REDSHIRT_3_PREPROD = {
  :aws_billing_id =>  '8',
  :owner_id => '781971168076',
  :name => 'Redshirt 3'
}
